//************************************************************************
// KS108.C
// LowLevel Routinen fr LCD-Displays mit KS0108
//
// Hier fr Displaytech 64240A
//
// Mit bis zu vier KS108 Chips also bis 240x64
// Ausgelegt bisher nur auf diesen Displaytyp
//
// Der Nullpunkt des Displays liegt oben,links. Fr unten rechts
// mssen dann zustzliche Umrechnungen stattfinden.
//
// Der KS108 zeichnet ein Byte VERTIKAL, nicht horizontal.
// 
// Y1 Y2 ...
// D0 D0 ...
// D1 D1 ...
// D2 D2 ...
// .. ..
// D7 D7 ...
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 26.02.2006
// Compiler: MCC18
//************************************************************************
#include <p18cxxx.h>
#include "ks108.h"
#include "fontsoft.h"
#include "delay.h"

unsigned char xcursor,ycursor;
unsigned char cgram[9];
unsigned char xpos,ypos;
unsigned char pixbuffer[96];
unsigned char pixlen;

unsigned char cgchars[256]; //32 chars 8 pixels width

//###################################################################################
// E ist die Bremse des Displays. Braucht ganz schn lange Delays.
// Geniale Angaben zum Timing im Datenblatt:
// 450ns Minimum fr E_low und E_high. Zusammen 1000ns. Fehlen schonmal 100ns.
// Und wo ist das Maximum ? Da ist raten angesagt.
void Wait_for_E_High(void)
//###################################################################################
{
//Anzahl NOP's fr 12MHz, ein NOP() 83ns
 NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();

 NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();
 NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();

//Immer noch nicht gut
 NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();
 NOP(); NOP(); NOP(); NOP();
 NOP(); NOP();
}

void Wait_for_E_Low(void)
//###################################################################################
{
//Anzahl NOP's fr 12MHz, ein NOP() 83ns
 NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();

 NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();
 NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();

//Immer noch nicht gut
 NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP(); NOP();
 NOP(); NOP(); NOP(); NOP();
 NOP(); NOP();
}

//###################################################################################
void WriteDisplay(unsigned char a)
//###################################################################################
{
 DATA_DIR_OUT();
 WRITE_DATA(a);
 E_ON();
 E_OFF();
}

//##########################################################
// Liest ein Byte aus dem Display
unsigned char ReadDisplay(void)
{
 unsigned char by;

 RS_ON();
 RW_ON(); //Wir wollen lesen

//Display mu zweimal gelesen werden !
//Dummy Read
 E_ON();
 E_OFF();

 DATA_DIR_IN();

 RS_ON();
 RW_ON(); //Wir wollen lesen

 E_ON();
 by=READ_DATA(); //DATA RAM lesen
 E_OFF();

 RW_OFF(); //zurck auf schreiben

 DATA_DIR_OUT();
 
 return by;
}

//##########################################################
//Pixel auf dem Display setzen
//mode=0; Pixel lschen
//mode>0; Pixel setzen
void SetPixel(unsigned char xpos, unsigned char ypos, unsigned char mode)
{
 unsigned char by,tmp,pixpos;
 unsigned char pixset;

 if(xpos>=LCD_WIDTH) return; //Punkt auerhalb der Anzeige
 if(ypos>=LCD_HEIGHT) return;

// if(mode==0) pixset=0;
// else pixset=1;
 pixset=mode; //Pixel gesetzt oder nicht erstmal merken

 SetPosition(xpos,ypos); //Position auf 8 Pixel genau setzen

 by=ReadDisplay();       //8 Pixel lesen

 tmp= ypos % 8; //Position des Pixels im Byte
 pixpos= 1 << tmp;             //An die richtige Position schieben

 if(pixset==1 && by&pixpos) return; //Pixel hatte den Wert schon
 if(pixset==0 && (by&pixpos)==0) return; //Pixel hatte den Wert schon

 if(pixset==0) by&=~pixpos; //Pixel lschen
 else by|=pixpos; //Pixel setzen

 SetPosition(xpos,ypos); //Position wird nach dem lesen verndert !
                         //Also neu setzen
 WriteDisplay(by);       //8 Pixel schreiben
}

//##########################################################
//An jeden KS0108 mu der DisplayOn Befehl gesendet werden
void InitDisplay(void)
{
 unsigned char result,i;

 INIT_CONTROL_PINS();

 RSTB_OFF();
 E_OFF();
 RW_OFF();
 Delay1ms(1);
 RSTB_ON();
 Delay1ms(100);

 for(i=0; i<MAX_CONTROLLERS; i++)
  {
   SetChipNumber(i); 

   RS_OFF(); //Instruction senden setzen
   WriteDisplay(0x3F); //Befehl Display On

// Tip von Olaf Schulz: Display Start Line bei seinen Displays nach Reset scheinbar nicht ok.
// Das Bild ist dann in Y-Richtung verschoben. Dies soll helfen.
   NOP(); NOP();
   WriteDisplay(0xC0); //Befehl Display Start Line 0 fr jeden Controller senden
    
   RS_ON(); //Zurck auf Daten
  } 

 SetFont(1);
 Delay1ms(100);
 ClearScreen();
}

//##########################################################
void ClearScreen(void)
//##########################################################
{
 unsigned char i,j,k;

 k=0;
 do
  {
   SetChipNumber(k); 

   for(j=0; j<8; j++)
    {
     SetPageNumber(j);
     RS_ON();
     RW_OFF(); //Display schreiben setzen

     WRITE_DATA(0);
     for(i=0; i<64; i++)
      { E_ON();
        E_OFF();
      }
    }

  k++;
 }while(k<MAX_CONTROLLERS);

 xcursor=0; //Reset textcursor position
 ycursor=0;

}

//##########################################################
void LCDChar(unsigned char ch)
//##########################################################
{
 if(ch<0x20) LCDCGChar(ch,xcursor,ycursor);
 else LCDSoftChar(ch,xcursor,ycursor);
 xcursor += FONT_PITCH;

 if(xcursor>=LCD_WIDTH)
  {
   xcursor=0;
   ycursor+=8;
   if(ycursor>=LCD_HEIGHT) ycursor=0;
  }
}

//##########################################################
void LCDPos(unsigned char row, unsigned char col)
//##########################################################
{
 xcursor=(col-1) * FONT_PITCH;
 ycursor=(row-1) * 8;
}

//##########################################################
void SetPosition(unsigned char x, unsigned char y)
//##########################################################
{
 SetChipNumber(x/64);
 SetPageNumber(y/8);
 SetRow(x);
}

//##########################################################
void MakeCGRam(void)
//##########################################################
{
 unsigned char by,mask,i,j,offset;
 
 offset=cgram[0]*8;

 for(i=0; i<8; i++) cgchars[i+offset]=0;

 //cgram[1] is the upper row of the char
 //MSB of the char is left
 for(i=0; i<8; i++) //resort the pixels for KS108
  {
   by=cgram[i+1];
   mask=0x80;
   
   for(j=0; j<8; j++)
    {
     if(by&mask) cgchars[j+offset]|=(1<<i);
     mask>>=1;
    } 
  } 
}

//##########################################################
void LCDCGChar(unsigned char ch, unsigned char xpos, unsigned char ypos)
//##########################################################
{
 unsigned char i,offset;

 offset=ch*8;
 
 SetPosition(xpos,ypos);
 for(i=0; i<8; i++) //ber die Breite des Zeichens
  {
   WriteDisplay(cgchars[i+offset]); //8 Pixel schreiben
  }//for i
}


