/*********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.40.01+
 * Company:
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Holger Klabunde      06/01/06    Modified CDC Demo
 *
 * 09.02.2006	Added command for setting user defined output pins
 *
 * 26.01.2006	Begin to remove buffering and use direct io.
 *
 * 24.01.2006	added SET_BITMAP command.
 *
 * 07.01.2006	added SET_GFXBUFFER command. it contains a buffersize.
 *              so you can write multiple bytes with pixel arrays.
 *              this reduces command overhead.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "typedefs.h"

#include "usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user.h"
#include "ks108.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
byte old_sw2;

byte nextstate; //contains NEXT state machine code for receiving 

char input_buffer[BUFFER_SIZE];
char output_buffer[BUFFER_SIZE];

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void InitializeUSART(void);
void BlinkUSBStatus(void);
BOOL Switch2IsPressed(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
 unsigned char i;
 
    mInitAllLEDs();
    mInitAllSwitches();
    old_sw2 = sw2;
    
    T0CON=0b11000100; //Prescaler 1/32, Timer 8 Bit, Timer on
    InitDisplay();
    LCDChar('U'); //sign of life ;)
    LCDChar('S'); //remove if you don't like it
    LCDChar('B');
    LCDChar('G');
    LCDChar('r');
    LCDChar('a');
    LCDChar('p');
    LCDChar('h');
    LCDChar('2');
    LCDPos(1,1);
    nextstate=NO_STATE; //Reset State Machine

}//end UserInit


/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
 byte ch,i,len,col,row,buffcount,fillcount,fillend,setport;
 byte by; //don't use this ! it's used by macro's !
 byte bmpheight,bmpyoffset;
 byte mask;
 
 unsigned int adress;
  
//remove this if you don't like two flashing LED's
    BlinkUSBStatus();


    // User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;


//#####################################################################
// Because we don't know if a command is completly in the input buffer
// or not, we have to use a state maschine here
//#####################################################################

   len=getsUSBUSART(input_buffer, BUFFER_SIZE); //do we have some data ?
//   len=getsUSBUSART(input_buffer); //do we have some data ?
   if(len>0)
    {
     buffcount=0;
     
     do
      {
       ch=input_buffer[buffcount++];

       if(nextstate!=NO_STATE)
        {
//This switch is sorted by priority !
         switch(nextstate)
          {
           case STATE_GETBITMAP_FULLSCREEN:
                if((xpos%64) == 0) { SetPosition(xpos,ypos); } //Position setzen
                WriteDisplay(ch);       //8 Pixel schreiben
                xpos++;
                
                while(buffcount<len-1 && xpos<LCD_WIDTH) //one buffcount++ is needed for 'continue'
                 {
                  ch=input_buffer[buffcount++];
                  if((xpos%64) == 0) { SetPosition(xpos,ypos); } //Position setzen
                  WriteDisplay(ch);       //8 Pixel schreiben
                  xpos++;

                 }
                   
                if(xpos>=LCD_WIDTH)
                 {
                  xpos=0;
                  ypos+=8; //next row

                  if(ypos >= bmpheight) //last row of bitmap drawn ?
                   {
                    nextstate=NO_STATE;
                   }

                 }

           break;

           case STATE_GETBITMAP_GETROW:
                pixbuffer[fillcount++]=ch;
/* not used

                while(fillcount<pixlen)
                 {
                  if(buffcount<len-1) //one buffcount++ is needed for 'continue'
                   {
                    pixbuffer[fillcount++]=input_buffer[buffcount++];
                   }
                  else break;
                 }

                if(fillcount>=pixlen) //plot the row
                 {
//-------------------------------------------------------------------------------
//                  SetPixelArray();
                  fillcount=0; //start next row
                  ypos++;       //next row position
                 }
                //else keep state

                if(ypos >= (bmpheight+bmpyoffset)) //last row of bitmap drawn ?
                 {
                  nextstate=NO_STATE;
                  fillcount=0;
                 }
*/
           break;
           case STATE_GFXBUFFER_FILL:
                pixbuffer[fillcount++]=ch;

/* not used
                while(fillcount<pixlen)
                 {
                  if(buffcount<len-1) //one buffcount++ is needed for 'continue'
                   {
                    pixbuffer[fillcount++]=input_buffer[buffcount++];
                   }
                  else break;
                 }

                if(fillcount>=pixlen)
                 {
//-------------------------------------------------------------------------------
//                  SetPixelArray();
                  nextstate=NO_STATE;
                 }
                //else keep state
*/
           break;
           
           case STATE_GETBITMAP_X: xpos=ch; nextstate=STATE_GETBITMAP_Y; break;
           case STATE_GETBITMAP_Y: ypos=ch; bmpyoffset=ch; nextstate=STATE_GETBITMAP_WIDTH; break;
           case STATE_GETBITMAP_WIDTH: pixlen=ch; nextstate=STATE_GETBITMAP_HEIGHT; break;
           case STATE_GETBITMAP_HEIGHT:
                bmpheight=ch;
                fillcount=0;

                if(pixlen==BYTES_PER_ROW && bmpheight==LCD_HEIGHT
                   && xpos==0 && ypos==0 ) //here comes a full screen bitmap !
                 {
                  nextstate=STATE_GETBITMAP_FULLSCREEN;
                 }
                else
                 { 
                  nextstate=STATE_GETBITMAP_GETROW; //receive data on one row only
                 }

           break;

           case STATE_GETGFXBUFFER_X: xpos=ch; nextstate=STATE_GETGFXBUFFER_Y; break;
           case STATE_GETGFXBUFFER_Y: ypos=ch; nextstate=STATE_GETGFXBUFFER_LEN; break;
           case STATE_GETGFXBUFFER_LEN:
                pixlen =ch;   //number of bytes in buffer
                fillcount = 0; //buffer index
                nextstate=STATE_GFXBUFFER_FILL;
           break;
                
           case STATE_GETGFX_X: xpos=ch; nextstate=STATE_GETGFX_Y; break;
           case STATE_GETGFX_Y: ypos=ch; nextstate=STATE_GETGFX_BYTE; break;
           case STATE_GETGFX_BYTE:
                pixbuffer[0]=ch;
/* not used
                pixlen=1;
//-------------------------------------------------------------------------------
//                SetPixelArray();
*/
                nextstate=NO_STATE;
           break;

           case STATE_GETCOL: col=ch; nextstate=STATE_GETROW; break;
           case STATE_GETROW: row=ch; LCDPos(row,col); nextstate=NO_STATE; break;
           case STATE_GETCGADR: cgram[0]=ch; nextstate=STATE_GETCG0; break;
           case STATE_GETCG0:   cgram[1]=ch; nextstate=STATE_GETCG1; break;
           case STATE_GETCG1:   cgram[2]=ch; nextstate=STATE_GETCG2; break;
           case STATE_GETCG2:   cgram[3]=ch; nextstate=STATE_GETCG3; break;
           case STATE_GETCG3:   cgram[4]=ch; nextstate=STATE_GETCG4; break;
           case STATE_GETCG4:   cgram[5]=ch; nextstate=STATE_GETCG5; break;
           case STATE_GETCG5:   cgram[6]=ch; nextstate=STATE_GETCG6; break;
           case STATE_GETCG6:   cgram[7]=ch; nextstate=STATE_GETCG7; break;
           case STATE_GETCG7:
                cgram[8]=ch;
        
                MakeCGRam(); //CG-RAM Daten setzen

                nextstate=NO_STATE;
           break;
           case STATE_GETCONTRAST:
                //contrast=ch; //nicht benutzt, nur Parameter abholen
                nextstate=NO_STATE;
           break;
           case STATE_GETBACKLIGHT:
                //backlight=ch; //nicht benutzt, nur Parameter abholen
                         //0=immer an, 1-100 = an fr X Minuten
                nextstate=NO_STATE;
           break;

           case STATE_GETOUTPUT_PORT:
                setport=ch;
                nextstate=STATE_GETOUTPUT_VALUE;
           break;

           case STATE_GETOUTPUT_VALUE:
                //we can only set free pins on PORTA, maybe more ports with future 18F4550 board !
                if(setport==0)
                 {
                  //NOTE: PORTA0,1 are changed by BlinkUSBStatus() !
//                  PORTA=ch;
                 }
                nextstate=NO_STATE;
           break;

           default: nextstate=NO_STATE; break;
          }//switch(nextstate)
          
         continue;
        }//if(nextstate!=NO_STATE)

       if( nextstate==NO_STATE && (ch<CLEAR_SCREEN || ch>LAST_COMMAND) )
        {
         LCDChar(ch); //Alles andere ist Text
         continue;
        }
       
       if( nextstate==NO_STATE && ch>=CLEAR_SCREEN && ch<LAST_COMMAND )
        {
         nextstate=STATE_COMMAND;  
        }
       
//Einige Befehle werden hier nur abgefangen bzw. deren Parameter
//abgeholt damit sie nicht als Text ausgegeben werden ! 
       if(nextstate==STATE_COMMAND)
        {

//This switch is sorted by priority !
         switch(ch)
          {
           case SET_GFXBUFFER: nextstate=STATE_GETGFXBUFFER_X; break;   //Pixel array adress X
           case SET_BITMAP: nextstate=STATE_GETBITMAP_X; break;   //start adress X
           case SET_GFX: nextstate=STATE_GETGFX_X; break;   //Pixel array adress X
           case SET_CURSOR: nextstate=STATE_GETCOL; break; //Go To Position, Col, Row
           case SET_CGCHAR: nextstate=STATE_GETCGADR; break; //Create Custom Character, 1 Byte Zeichennummer (0-7), 8 Databytes
           case CLEAR_SCREEN: ClearScreen(); nextstate=NO_STATE; break; //Clear Display
           case CURSOR_HOME: LCDPos(1,1); nextstate=NO_STATE; break; //Go To Top Left
           case SET_CONTRAST: nextstate=STATE_GETCONTRAST; break; //Contrast, Parameter abholen
           case SET_LIGHT: nextstate=STATE_GETBACKLIGHT; break; //Backlight On, Parameter abholen
           case SET_OUTPUTS: nextstate=STATE_GETOUTPUT_PORT; break;   //Port to set pins
           default: nextstate=NO_STATE; break;
          }//switch ch
          
         continue;
        }//if(nextstate==STATE_COMMAND)
        
     }while(buffcount<len);

/*
     if(mUSBUSARTIsTxTrfReady()) //Echo back complete buffer
      {
       mUSBUSARTTxRam((byte *)input_buffer, len);
      }
*/
    }//if(len>0) 

}//end ProcessIO




/******************************************************************************
 * Function:        void BlinkUSBStatus(void)
 * Overview:        BlinkUSBStatus turns on and off LEDs corresponding to
 *                  the USB device state.
 *
 * Note:            mLED macros can be found in io_cfg.h
 *                  usb_device_state is declared in usbmmap.c and is modified
 *                  in usbdrv.c, usbctrltrf.c, and usb9.c
 *****************************************************************************/
void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(UCONbits.SUSPND == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            mLED_2 = mLED_1;        // Both blink at the same time
        }//end if
    }
    else
    {
        if(usb_device_state == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(usb_device_state == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(usb_device_state == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(usb_device_state == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(usb_device_state == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(usb_device_state == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
                mLED_2 = !mLED_1;       // Alternate blink                
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus

BOOL Switch2IsPressed(void)
{
    if(sw2 != old_sw2)
    {
        old_sw2 = sw2;                  // Save new value
        if(sw2 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch2IsPressed


/** EOF user.c ***************************************************************/
